# -*- coding: utf8 -*-

# Demonstration script showing use of different recipes to measure a set of images. 
# This example applies recipes selectively based on the image filename.  
# For each recipe, there is a grep search string. 
# For each image in the project, if this search string is found in the image filename, than that recipe is run.
# As written, if an image name matches multiple search strings, then all matching recipes will be run on that image.
# If no search string matches, then no recipe is run on that image.  
# If measurements already exist, the images will be re-measured. 

# Import classes from the ProSEMpy scripting interface, and other libraries that will be used
from ProSEMpy import GProSEM
import sys
import re
import os

#   ==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++
#   ==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++

#    Before using this script on your own files, 
#    these must be edited to match your specifics:
#    (a) recipe location, and 
#    (b) the recipe names and corresponding filename match strings
#      

# Define this path to where the recipes are stored. 
#       note that the 'r' in front of the string treats this as a raw string, so the backslash characters are
#       handled literally, not as escaping characters

RecipePath = r"C:\Users\rbojk\Documents\ProSEM\Website\In Action examples\Scripts-SelectiveRecipes"

# This list hold tuple pairs:   Recipe Name , Match String
Recipes = [ ("ZEP_DEN_000","DEN_000"),
            ("ZEP_DEN_025","DEN_025"),
            ("ZEP_DEN_050","DEN_050"),
            ("ZEP_DEN_075","DEN_075"),
            ("ZEP_DEN_100","DEN_100"),
            ]

#   ==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++
#   ==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++


# Processing method, which can also be used from other scripts
# importing this one as a library.
def process(prosem):
    '''
    Applies multiple recipes to images in a project based on a string match (grep) to the image filename. 
    '''
    print("Number of Recipes to test and possibly apply: ",len(Recipes))
    print("Number of Images to possibly measure:",len(prosem))

    recipeCount =0
    for RecipeName,RecipeMatch in Recipes:
        recipeCount += 1
        ThisRecipe = os.path.join(RecipePath, RecipeName + '.mpr')
        print("Recipe {0}/{1}: {2!r}...".format(recipeCount,len(Recipes),RecipeName))
        prosem.load_recipe(ThisRecipe)
        for image in prosem:
            if re.search(RecipeMatch,image.label):        # is the match string in this image filename?
                print("   Image: {0!r}...".format(image.label))
                prosem.apply_recipe(image)
    print("Done.", file=sys.stderr)
    return True    # must return True so that the new measurements will be passed back into the interactive ProSEM instance. 



# When called as a program, load a project passed on the command line,
# process it, then save it back to the original file. 
if __name__ == '__main__':
    import os
    import traceback
    pause = True
    error = 1
    try:
        prosem = GProSEM()
        project = sys.argv[1]
        print("Loading {0!r}...".format(project), end=" ", file=sys.stderr)
        prosem.load_project(project)
        print("{0} images".format(len(prosem)), file=sys.stderr)
        os.chdir(prosem.base)
        if process(prosem):
            #print("Saving {0!r}...".format(prosem.file), end=" ", file=sys.stderr)
            prosem.save_project()
            #print("{0} images".format(len(prosem)), file=sys.stderr)
            pause = True
        error = 0
    except KeyboardInterrupt:
        pause = False
    except:
        traceback.print_exc()
    if pause:
        print("Press enter to continue...", file=sys.stderr)
        sys.stdin.read(1)
    sys.exit(error)
