# -*- coding: utf8 -*-

# Demonstration script showing use of different recipes to measure a set of images. 
# This example applies recipes selectively based on the image magnification, obtained from image metadata image filename.  

# For each recipe, there is defined magnification.
# There is also a magnifcation tolerance to allow for some slight variation in reported magnification found in some SEM models. 
# If measurements already exist, the images are not re-measured. 
#
# This example steps through each image in the project, and for each image, checks each recipe to see if the magnification matches
# within the tolerance. If it matches, then the corresponding recipe is run. 
# If no magnifcation matches, then no recipe is run on that image.  
#
# The recipes are expected to be in the same folder as the project file. 

# Import classes from the ProSEMpy scripting interface:
from ProSEMpy import GProSEM
import sys
import re
import os


#   ==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++
#   ==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++
#    Before using this script on your own files, 
#    these must be edited to match your specifics:
#    (a) the recipe names and corresponding magnifications to be matched
#    (b) the key of the image metadata item which contains the image magnification to match.  
#       This varies for each SEM manufacturer and software platform.  The available metadata tags for an image can be
#       viewed in ProSEM's Image Information panel.      

    # This list hold tuple pairs:   Recipe Name , Magnification
    # These recipes much be in the directory pointed to by RecipePath
Recipes = [ ("EBET018 dot array 16kx","16000"),
            ("EBET018 dot array 30kx","30000")
            ]

metadataKey = '$CM_MAG'         # the metadata key to match the value above; must match exactly, including case
                                    # the metadata keys available depend on the SEM software and settings. 

toleranceMagnification = 0.05   # if the magnification matches within this tolerance, then that recipe is used.
                                    #  0.05 = 5% of mag total range, so +/- 2.5% above or below.

#   ==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++
#   ==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++


# Processing method, which can also be used from other scripts
# importing this one as a library.
def process(prosem):
    '''
    Applies recipes selectively to images in a project based on a metadata value. 
    '''

    RecipePath = prosem.base
    
    if len(prosem) == 0:
        print ('There are no images in this project.')
        return

    # check that recipes exist first...
    for RecipeName,RecipeMag in Recipes:
        ThisRecipe = os.path.join(RecipePath, RecipeName + '.mpr')
        recipesOK = True
        if not os.path.isfile(ThisRecipe):  #does the recipe file exist in the right place?
            print("\nError:\nRecipe: {}\nnot found at: {}".format(RecipeName,RecipePath))
            recipesOK = False
    if not recipesOK:
        return

    for image in prosem:
        if len(image) > 0:
            print('Image {}:  Already has stored measurements. Image not remeasured.'.format(image.label))
            continue     # go on to the next image
        try:        # use try/except to catch any errors that happen trying to get the metadata value, most probably that it just isn't there.
            thisMag = float(image.metadata[metadataKey])   # get the magnification, and convert it to a floating point number
        except:
            print ('Image {}:  Metadata item not found: {}.  Image not measured. '. format(image.label,metadataKey))
            continue     # go on to the next image
        for RecipeName,RecipeMag in Recipes:   # loop through the recipe, load and run the first one that matches
            testCriterion = (abs(float(RecipeMag) - thisMag)/ float(RecipeMag)) <= toleranceMagnification
            if (abs(float(RecipeMag) - thisMag)/ float(RecipeMag)) <= toleranceMagnification:
                print('Image {}:  Measuring with recipe: {}'.format(image.label,RecipeName))
                ThisRecipe = os.path.join(RecipePath, RecipeName + '.mpr')
                prosem.load_recipe(ThisRecipe)
                prosem.apply_recipe(image)
                break        # only run the first matching recipe, so once measured, go on to next image. 
        if len(image) == 0:
            print('Image {}:  No matching criterion for image mag: {} found.  Image not measured.'.format(image.label,thisMag))
    print("Done.", file=sys.stderr)
    return True    # must return True so that the new measurements will be passed back into the interactive ProSEM instance. 



# When called as a program, load a project passed on the command line,
# process it, then save it back to the original file:
if __name__ == '__main__':
    import os
    import traceback
    pause = True
    error = 1
    try:
        prosem = GProSEM()
        project = sys.argv[1]
        print("Loading {0!r}...".format(project), end=" ", file=sys.stderr)
        prosem.load_project(project)
        print("{0} images".format(len(prosem)), file=sys.stderr)
        os.chdir(prosem.base)
        if process(prosem):
            #print("Saving {0!r}...".format(prosem.file), end=" ", file=sys.stderr)
            prosem.save_project()
            #print("{0} images".format(len(prosem)), file=sys.stderr)
            pause = True
        error = 0
    except KeyboardInterrupt:
        pause = False
    except:
        traceback.print_exc()
    if pause:
        print("Press enter to continue...", file=sys.stderr)
        sys.stdin.read(1)
    sys.exit(error)
